'use client'

import React, { useState, useEffect } from 'react';
import FHIR from 'fhirclient';
import Client from 'fhirclient/lib/Client';

const ConsentManagement = () => {
  const [client, setClient] = useState<Client | null>(null);
  const [patientId, setPatientId] = useState('');
  const [nameSearch, setNameSearch] = useState('');
  const [patients, setPatients] = useState<Array<{ resource: any }>>([]);
  const [consents, setConsents] = useState([]);
  const [selectedConsent, setSelectedConsent] = useState(null);
  const [operationOutcome, setOperationOutcome] = useState(null);
  const [selectedPatient, setSelectedPatient] = useState(null);
  const [consentType, setConsentType] = useState('deny-all');
  const [organizationSearch, setOrganizationSearch] = useState('');
  const [organizations, setOrganizations] = useState([]);
  const [selectedOrganization, setSelectedOrganization] = useState(null);
  const [practitionerSearch, setPractitionerSearch] = useState('');
  const [practitioners, setPractitioners] = useState([]);
  const [selectedPractitioner, setSelectedPractitioner] = useState(null);
  const [allOrganizations, setAllOrganizations] = useState([]);
  const [allPractitioners, setAllPractitioners] = useState([]);
  const [username, setUsername] = useState<string>('');

  const parseOperationOutcome = (error) => {
    try {
      // If error is already an OperationOutcome
      if (error?.resourceType === 'OperationOutcome') {
        return error;
      }

      // Extract JSON from HTTP error message
      let errorText = error.message;
      if (errorText.includes('URL:')) {
        // Extract everything after "URL:" and before the first '{'
        const urlPart = errorText.split('URL:')[1];
        const jsonStartIndex = urlPart.indexOf('{');
        if (jsonStartIndex !== -1) {
          errorText = urlPart.slice(jsonStartIndex);
        }
      }

      try {
        const parsed = JSON.parse(errorText);
        if (parsed.resourceType === 'OperationOutcome') {
          return parsed;
        }
      } catch (parseError) {
        // Continue to next parsing attempt if JSON.parse fails
      }

      // Try to find any JSON object in the error message
      const jsonMatch = errorText.match(/\{[^]*\}/);
      if (jsonMatch) {
        try {
          const parsed = JSON.parse(jsonMatch[0]);
          if (parsed.resourceType === 'OperationOutcome') {
            return parsed;
          }
        } catch (parseError) {
          // Continue to fallback if JSON parsing fails
        }
      }

      // If no OperationOutcome found, create a generic one
      return {
        resourceType: 'OperationOutcome',
        issue: [{
          severity: 'error',
          code: 'processing',
          diagnostics: errorText
        }]
      };
    } catch (e) {
      // If all parsing fails, return generic error
      return {
        resourceType: 'OperationOutcome',
        issue: [{
          severity: 'error',
          code: 'processing',
          diagnostics: error.message || 'An unknown error occurred'
        }]
      };
    }
  };

  useEffect(() => {
    const init = async () => {
      try {
        const fhirClient = await FHIR.oauth2.ready();
        
        // Extract username from token
        const tokenResponse = fhirClient.getState().tokenResponse;
        if (tokenResponse?.access_token) {
          const payload = tokenResponse.access_token.split('.')[1];
          const decodedPayload = JSON.parse(atob(payload));
          setUsername(decodedPayload.sub || '');
        }

        setClient(fhirClient);
      } catch (err) {
        setClient(null);
        setUsername('');
      }
    };

    init();
  }, []);

  const initiateSmartAuth = () => {
    FHIR.oauth2.authorize({
      clientId: 'consentdemo',
      scope: 'online_access openid patient/*.read patient/*.write cdr_fhir_delete',
      redirectUri: 'http://localhost:3000/callback',
      iss: 'http://localhost:8000/'
    });
  };

  const searchConsents = async () => {
    if (!patientId || !client) {
      setOperationOutcome({
        resourceType: 'OperationOutcome',
        issue: [{
          severity: 'error',
          code: 'processing',
          diagnostics: 'No patient selected'
        }]
      });
      return;
    }

    try {
      const response = await client.request(`Consent?patient=${patientId}`);
      setConsents(response.entry || []);
      setOperationOutcome(null);
    } catch (err) {
      setOperationOutcome(parseOperationOutcome(err));
    }
  };

  const handlePatientSelect = async (patient) => {
    setPatientId(patient.id);
    setSelectedPatient(patient);
    try {
      const response = await client.request(`Consent?patient=${patient.id}`);
      setConsents(response.entry || []);
      setPatients([]); // Clear results after selection
      setOperationOutcome(null);
    } catch (err) {
      setOperationOutcome(parseOperationOutcome(err));
    }
  };

  const searchOrganizations = async (searchText) => {
    if (!client || !searchText.trim()) return;
    try {
      const response = await client.request(`Organization?name=${searchText}`);
      setOrganizations(response.entry || []);
    } catch (err) {
      setOperationOutcome(parseOperationOutcome(err));
    }
  };

  const searchPractitioners = async (searchText) => {
    if (!client || !searchText.trim()) return;
    try {
      const response = await client.request(`Practitioner?name=${searchText}`);
      setPractitioners(response.entry || []);
    } catch (err) {
      setOperationOutcome(parseOperationOutcome(err));
    }
  };

  const createConsent = async () => {
    if (!client) return;

    let startDate = new Date().toISOString();
    let provision = {
      type: "deny",
      period: {
        start: startDate
      }
    };
    let category = {
      coding: [{
        system: "http://loinc.org",
        code: "59284-0",
        display: "Patient Consent"
      }]
    };

    // Customize provision based on consent type
    switch (consentType) {
      case 'organization':
        if (!selectedOrganization) {
          setOperationOutcome({
            resourceType: 'OperationOutcome',
            issue: [{
              severity: 'error',
              code: 'processing',
              diagnostics: 'Please select an organization'
            }]
          });
          return;
        }
        provision = {
          type: "permit",
          period: { start: startDate },
          actor: [{
            role: {
              coding: [{
                system: "http://terminology.hl7.org/CodeSystem/v3-ParticipationType",
                code: "PRCP"
              }]
            },
            reference: {
              reference: `Organization/${selectedOrganization.resource.id}`
            }
          }]
        };
        category = {
          coding: [{
            system: "http://terminology.hl7.org/CodeSystem/v3-ActCode",
            code: "IDSCL",
            display: "information disclosure"
          }]
        };
        break;

      case 'practitioner':
        if (!selectedPractitioner) {
          setOperationOutcome({
            resourceType: 'OperationOutcome',
            issue: [{
              severity: 'error',
              code: 'processing',
              diagnostics: 'Please select a practitioner'
            }]
          });
          return;
        }
        provision = {
          type: "permit",
          period: { start: startDate },
          actor: [{
            role: {
              coding: [{
                system: "http://terminology.hl7.org/CodeSystem/v3-ParticipationType",
                code: "PRCP"
              }]
            },
            reference: {
              reference: `Practitioner/${selectedPractitioner.resource.id}`
            }
          }]
        };
        category = {
          coding: [{
            system: "http://terminology.hl7.org/CodeSystem/v3-ActCode",
            code: "INFA",
            display: "information access"
          }]
        };
        break;
    }

    const newConsent = {
      resourceType: 'Consent',
      status: 'active',
      patient: {
        reference: `Patient/${patientId}`
      },
      dateTime: startDate,
      scope: {
        coding: [{
          system: 'http://terminology.hl7.org/CodeSystem/consentscope',
          code: 'patient-privacy',
          display: 'Privacy Consent'
        }]
      },
      category,
      provision
    };

    try {
      const response = await client.create(newConsent);
      setConsents([...consents, { resource: response }]);
      setOperationOutcome(null);
      // Reset selection states
      setConsentType('deny-all');
      setSelectedOrganization(null);
      setSelectedPractitioner(null);
    } catch (err) {
      setOperationOutcome(parseOperationOutcome(err));
    }
  };

  const updateConsent = async (consent) => {
    if (!client) return;

    try {
      const updatedConsent = await client.update(consent);
      setConsents(consents.map(c =>
        c.resource.id === consent.id ? { resource: updatedConsent } : c
      ));
      setOperationOutcome(null);
    } catch (err) {
      setOperationOutcome(parseOperationOutcome(err));
    }
  };

  const deleteConsent = async (consentId) => {
    if (!client) return;

    try {
      // Delete the consent using the correct URL format
      await client.delete(`Consent/${consentId}`);

      // Update the UI
      setConsents(consents.filter(c => c.resource.id !== consentId));
      setOperationOutcome(null);
    } catch (err) {
      console.error('Delete error:', err);
      setOperationOutcome(parseOperationOutcome(err));
    }
  };

  const searchPatients = async () => {
    if (!client || !nameSearch.trim()) return;
    setSelectedPatient(null);
    try {
      const response = await client.request(`Patient?name=${nameSearch}`);
      setPatients(response.entry || []);
      setOperationOutcome(null);
    } catch (err) {
      setOperationOutcome(parseOperationOutcome(err));
    }
  };

  const handleLogout = async () => {
    if (!client) return;

    try {
      const state = client.getState();
      const token = state.tokenResponse?.access_token;

      if (token) {
        const revokeUrl = new URL('/oauth2/revoke', state.serverUrl).href;
        await fetch(revokeUrl, {
          method: 'POST',
          headers: {
            'Content-Type': 'application/x-www-form-urlencoded'
          },
          body: `token=${token}`
        });
      }
    } catch (err) {
      // Silently continue with logout even if token revocation fails
    } finally {
      // Reset component state
      setClient(null);
      setUsername('');
      setPatientId('');
      setNameSearch('');
      setPatients([]);
      setConsents([]);
      setSelectedConsent(null);
      setOperationOutcome(null);
      setSelectedPatient(null);

      // Clear any stored SMART state
      localStorage.clear();
      sessionStorage.clear();

      // Redirect to home
      window.location.href = '/';
    }
  };

  // Add helper function to determine consent type and details
  const getConsentDetails = (consent) => {
    const provision = consent.provision;
    if (!provision?.actor?.length) {
      return {
        type: 'deny-all',
        bgColor: 'bg-red-50',
        borderColor: 'border-red-200',
        textColor: 'text-red-700',
        label: 'Deny All Access',
        detail: null
      };
    }

    const actor = provision.actor[0];
    const reference = actor.reference.reference;

    if (reference.startsWith('Organization/')) {
      return {
        type: 'organization',
        bgColor: 'bg-blue-50',
        borderColor: 'border-blue-200',
        textColor: 'text-blue-700',
        label: 'Organization Access',
        detail: reference
      };
    }

    if (reference.startsWith('Practitioner/')) {
      return {
        type: 'practitioner',
        bgColor: 'bg-green-50',
        borderColor: 'border-green-200',
        textColor: 'text-green-700',
        label: 'Practitioner Access',
        detail: reference
      };
    }

    return {
      type: 'unknown',
      bgColor: 'bg-gray-50',
      borderColor: 'border-gray-200',
      textColor: 'text-gray-700',
      label: 'Unknown Type',
      detail: null
    };
  };

  // Add function to fetch all organizations
  const fetchAllOrganizations = async () => {
    if (!client) return;
    try {
      const response = await client.request('Organization');
      setAllOrganizations(response.entry || []);
    } catch (err) {
      setOperationOutcome(parseOperationOutcome(err));
    }
  };

  // Add effect to fetch organizations when consent type changes to organization
  useEffect(() => {
    if (consentType === 'organization' && client) {
      fetchAllOrganizations();
    }
  }, [consentType, client]);

  // Add function to fetch all practitioners
  const fetchAllPractitioners = async () => {
    if (!client) return;
    try {
      const response = await client.request('Practitioner');
      setAllPractitioners(response.entry || []);
    } catch (err) {
      setOperationOutcome(parseOperationOutcome(err));
    }
  };

  // Add effect to fetch practitioners when consent type changes
  useEffect(() => {
    if (consentType === 'practitioner' && client) {
      fetchAllPractitioners();
    }
  }, [consentType, client]);

  return (
    <div className="p-4 max-w-4xl mx-auto">
      <div className="bg-white shadow rounded-lg">
        <div className="p-6">
          <div className="flex justify-between items-center mb-6">
            <h1 className="text-2xl font-bold">Smile Demo Consent Management</h1>
            {client && (
              <div className="flex items-center gap-4">
                <span className="text-gray-600">
                  {username}
                </span>
                <button
                  onClick={handleLogout}
                  className="bg-red-500 text-white px-4 py-2 rounded hover:bg-red-600 focus:outline-none focus:ring-2 focus:ring-red-500 focus:ring-offset-2"
                >
                  Logout
                </button>
              </div>
            )}
          </div>

          {!client ? (
            <button
              onClick={initiateSmartAuth}
              className="bg-blue-500 text-white px-4 py-2 rounded hover:bg-blue-600 focus:outline-none focus:ring-2 focus:ring-blue-500 focus:ring-offset-2"
            >
              Login with SMART
            </button>
          ) : (
            <div className="space-y-6">
              <div className="flex gap-4">
                <form onSubmit={(e) => {
                  e.preventDefault();
                  searchPatients();
                }} className="flex gap-4 w-full">
                  <input
                    type="text"
                    placeholder="Search patient by name"
                    value={nameSearch}
                    onChange={(e) => setNameSearch(e.target.value)}
                    className="flex-1 border rounded px-3 py-2"
                  />
                  <button type="submit" className="bg-blue-500 text-white px-4 py-2 rounded">
                    Search Patients
                  </button>
                </form>
              </div>

              {selectedPatient && (
                <div className="bg-blue-50 border border-blue-200 rounded-lg p-4">
                  <div className="flex justify-between items-center">
                    <div>
                      <p className="text-sm text-blue-600">Selected Patient</p>
                      <p className="font-medium">
                        {selectedPatient.name?.[0]?.given?.join(' ')} {selectedPatient.name?.[0]?.family}
                      </p>
                      <p className="text-sm text-gray-500">ID: {selectedPatient.id}</p>
                    </div>
                    <button
                      onClick={() => {
                        setSelectedPatient(null);
                        setPatientId('');
                        setConsents([]);
                      }}
                      className="text-sm text-blue-600 hover:text-blue-800"
                    >
                      Change Patient
                    </button>
                  </div>
                </div>
              )}

              {patients.length > 0 && (
                <div className="border rounded-lg divide-y">
                  {patients.map(({ resource: patient }) => (
                    <div
                      key={patient.id}
                      onClick={() => handlePatientSelect(patient)}
                      className="p-4 hover:bg-gray-50 cursor-pointer"
                    >
                      <p className="font-medium">
                        {patient.name?.[0]?.given?.join(' ')} {patient.name?.[0]?.family}
                      </p>
                      <p className="text-sm text-gray-500">ID: {patient.id}</p>
                    </div>
                  ))}
                </div>
              )}

              {selectedPatient && (
                <div className="space-y-4">
                  <div className="border rounded-lg p-4">
                    <h3 className="text-lg font-medium mb-4">Create New Consent</h3>

                    <div className="space-y-4">
                      <div>
                        <label className="block text-sm font-medium text-gray-700 mb-1">
                          Consent Type
                        </label>
                        <select
                          value={consentType}
                          onChange={(e) => {
                            setConsentType(e.target.value);
                            setSelectedOrganization(null);
                            setSelectedPractitioner(null);
                          }}
                          className="w-full border rounded px-3 py-2"
                        >
                          <option value="deny-all">Deny All Access</option>
                          <option value="organization">Organization-specific</option>
                          <option value="practitioner">Practitioner-specific</option>
                        </select>
                      </div>

                      {consentType === 'organization' && (
                        <div className="space-y-2">
                          <label className="block text-sm font-medium text-gray-700">
                            Select Organization
                          </label>
                          <select
                            value={selectedOrganization ? selectedOrganization.resource.id : ''}
                            onChange={(e) => {
                              const org = allOrganizations.find(org =>
                                org.resource.id === e.target.value
                              );
                              setSelectedOrganization(org);
                            }}
                            className="w-full border rounded px-3 py-2"
                          >
                            <option value="">Select an organization...</option>
                            {allOrganizations.map(({ resource: org }) => (
                              <option key={org.id} value={org.id}>
                                {org.name}
                              </option>
                            ))}
                          </select>
                          {selectedOrganization && (
                            <div className="mt-2 p-2 bg-blue-50 rounded">
                              Selected: {selectedOrganization.resource.name}
                            </div>
                          )}
                        </div>
                      )}

                      {consentType === 'practitioner' && (
                        <div className="space-y-2">
                          <label className="block text-sm font-medium text-gray-700">
                            Select Practitioner
                          </label>
                          <select
                            value={selectedPractitioner ? selectedPractitioner.resource.id : ''}
                            onChange={(e) => {
                              const pract = allPractitioners.find(p =>
                                p.resource.id === e.target.value
                              );
                              setSelectedPractitioner(pract);
                            }}
                            className="w-full border rounded px-3 py-2"
                          >
                            <option value="">Select a practitioner...</option>
                            {allPractitioners.map(({ resource: pract }) => (
                              <option key={pract.id} value={pract.id}>
                                {pract.name?.[0]?.given?.join(' ')} {pract.name?.[0]?.family}
                              </option>
                            ))}
                          </select>
                          {selectedPractitioner && (
                            <div className="mt-2 p-2 bg-blue-50 rounded">
                              Selected: {selectedPractitioner.resource.name?.[0]?.given?.join(' ')} {selectedPractitioner.resource.name?.[0]?.family}
                            </div>
                          )}
                        </div>
                      )}

                      <button
                        onClick={createConsent}
                        className="bg-green-500 text-white px-4 py-2 rounded hover:bg-green-600"
                      >
                        Create Consent
                      </button>
                    </div>
                  </div>
                </div>
              )}

              {operationOutcome && (
                <div className="space-y-2">
                  {operationOutcome.issue.map((issue, index) => {
                    const severityColors = {
                      fatal: 'bg-red-100 border-red-500 text-red-900',
                      error: 'bg-red-50 border-red-400 text-red-700',
                      warning: 'bg-yellow-50 border-yellow-400 text-yellow-700',
                      information: 'bg-blue-50 border-blue-400 text-blue-700'
                    };

                    const severityIcons = {
                      fatal: '⛔',
                      error: '❌',
                      warning: '⚠️',
                      information: 'ℹ️'
                    };

                    return (
                      <div
                        key={index}
                        className={`p-4 rounded border ${severityColors[issue.severity] || severityColors.error}`}
                      >
                        <div className="flex items-start">
                          <span className="mr-2" role="img" aria-label={issue.severity}>
                            {severityIcons[issue.severity]}
                          </span>
                          <div>
                            <p className="font-medium capitalize">
                              {issue.severity} {issue.code && `(${issue.code})`}
                            </p>
                            <p className="mt-1">{issue.diagnostics}</p>
                          </div>
                        </div>
                      </div>
                    );
                  })}
                </div>
              )}

              <div className="space-y-4">
                {selectedConsent && (
                  <div className="fixed inset-0 bg-black bg-opacity-50 flex items-center justify-center p-4">
                    <div className="bg-white rounded-lg p-6 max-w-2xl w-full">
                      <h2 className="text-xl font-bold mb-4">Edit Consent</h2>
                      <div className="space-y-4">
                        <div>
                          <label className="block text-sm font-medium text-gray-700 mb-1">
                            Status
                          </label>
                          <select
                            value={selectedConsent.status}
                            onChange={(e) => setSelectedConsent({
                              ...selectedConsent,
                              status: e.target.value
                            })}
                            className="w-full border rounded px-3 py-2"
                          >
                            <option value="active">Active</option>
                            <option value="inactive">Inactive</option>
                            <option value="entered-in-error">Entered in Error</option>
                          </select>
                        </div>

                        {selectedConsent.provision?.actor?.length > 0 && (
                          <div>
                            <label className="block text-sm font-medium text-gray-700 mb-1">
                              Access Type
                            </label>
                            <select
                              value={selectedConsent.provision.type}
                              onChange={(e) => setSelectedConsent({
                                ...selectedConsent,
                                provision: {
                                  ...selectedConsent.provision,
                                  type: e.target.value
                                }
                              })}
                              className="w-full border rounded px-3 py-2"
                            >
                              <option value="permit">Permit Access</option>
                              <option value="deny">Deny Access</option>
                            </select>
                          </div>
                        )}

                        <div className="flex justify-end space-x-2">
                          <button
                            onClick={() => setSelectedConsent(null)}
                            className="px-4 py-2 border rounded hover:bg-gray-50"
                          >
                            Cancel
                          </button>
                          <button
                            onClick={() => {
                              updateConsent(selectedConsent);
                              setSelectedConsent(null);
                            }}
                            className="px-4 py-2 bg-blue-500 text-white rounded hover:bg-blue-600"
                          >
                            Save Changes
                          </button>
                        </div>
                      </div>
                    </div>
                  </div>
                )}

                {consents.map(({ resource: consent }) => {
                  const details = getConsentDetails(consent);
                  return (
                    <div
                      key={consent.id}
                      className={`border rounded-lg p-4 hover:shadow-md transition-shadow ${details.bgColor} ${details.borderColor}`}
                    >
                      <div className="flex justify-between items-start">
                        <div>
                          <div className={`font-medium ${details.textColor} mb-1`}>
                            {details.label}
                          </div>
                          <p className="text-sm text-gray-600">ID: {consent.id}</p>
                          <p className="text-sm text-gray-600">Status: {consent.status}</p>
                          <p className="text-sm text-gray-600">
                            Date: {new Date(consent.dateTime).toLocaleDateString()}
                          </p>
                          {details.detail && (
                            <p className="text-sm text-gray-600">
                              {details.detail}
                            </p>
                          )}
                          <p className={`text-sm mt-1 font-medium ${details.textColor}`}>
                            {consent.provision?.type === 'deny' ? 'Deny' : 'Permit'} Access
                          </p>
                        </div>
                        <div className="space-x-2">
                          <button
                            onClick={() => setSelectedConsent(consent)}
                            className="bg-gray-500 text-white px-3 py-1 rounded hover:bg-gray-600 focus:outline-none focus:ring-2 focus:ring-gray-500 focus:ring-offset-2"
                          >
                            Edit
                          </button>
                          <button
                            onClick={() => deleteConsent(consent.id)}
                            className="bg-red-500 text-white px-3 py-1 rounded hover:bg-red-600 focus:outline-none focus:ring-2 focus:ring-red-500 focus:ring-offset-2"
                          >
                            Delete
                          </button>
                        </div>
                      </div>
                    </div>
                  );
                })}
              </div>
            </div>
          )}
        </div>
      </div>
    </div>
  );
};

export default ConsentManagement;
