'use client'

import React, { useState, useEffect } from 'react';
import FHIR from 'fhirclient';
import Client from 'fhirclient/lib/Client';

interface Patient {
  id: string;
  name?: Array<{
    given?: string[];
    family?: string;
  }>;
}

interface Observation {
  id: string;
  code: {
    coding?: Array<{
      system?: string;
      code?: string;
      display?: string;
    }>;
    text?: string;
  };
  valueQuantity?: {
    value?: number;
    unit?: string;
  };
  valueString?: string;
  valueCodeableConcept?: {
    coding?: Array<{
      display?: string;
    }>;
    text?: string;
  };
  effectiveDateTime?: string;
  status: string;
}

const PatientViewer = () => {
  const [client, setClient] = useState<Client | null>(null);
  const [nameSearch, setNameSearch] = useState('');
  const [patients, setPatients] = useState<Array<{ resource: Patient }>>([]);
  const [selectedPatient, setSelectedPatient] = useState<Patient | null>(null);
  const [observations, setObservations] = useState<Array<{ resource: Observation }>>([]);
  const [operationOutcome, setOperationOutcome] = useState(null);
  const [username, setUsername] = useState<string>('');

  useEffect(() => {
    const init = async () => {
      try {
        console.log('Checking for existing SMART session...');
        const fhirClient = await FHIR.oauth2.ready();
        console.log('Found existing session:', fhirClient);
        
        // Extract username from token
        const tokenResponse = fhirClient.getState().tokenResponse;
        if (tokenResponse?.access_token) {
          const payload = tokenResponse.access_token.split('.')[1];
          const decodedPayload = JSON.parse(atob(payload));
          setUsername(decodedPayload.sub || '');
        }

        setClient(fhirClient);
      } catch (err) {
        console.log('No active session:', err);
        // Clear any stale state
        setClient(null);
        setUsername('');
      }
    };

    init();
  }, []);

  const parseOperationOutcome = (error: any) => {
    try {
      if (error?.resourceType === 'OperationOutcome') {
        return error;
      }

      return {
        resourceType: 'OperationOutcome',
        issue: [{
          severity: 'error',
          code: 'processing',
          diagnostics: error.message || 'An unknown error occurred'
        }]
      };
    } catch (e) {
      return {
        resourceType: 'OperationOutcome',
        issue: [{
          severity: 'error',
          code: 'processing',
          diagnostics: 'An unknown error occurred'
        }]
      };
    }
  };

  const initiateSmartAuth = () => {
    FHIR.oauth2.authorize({
      clientId: 'consentdemo',
      scope: 'online_access openid patient/*.read',
      redirectUri: 'http://localhost:3001/callback',
      iss: 'http://localhost:8000/'
    });
  };

  const handlePatientSelect = async (patient: Patient) => {
    setSelectedPatient(patient);
    try {
      const response = await client?.request(`Observation?patient=${patient.id}`);
      setObservations(response?.entry || []);
      setPatients([]); // Clear search results after selection
      setOperationOutcome(null);
    } catch (err) {
      setOperationOutcome(parseOperationOutcome(err));
    }
  };

  const searchPatients = async () => {
    if (!client || !nameSearch.trim()) return;
    setSelectedPatient(null);
    try {
      const response = await client.request(`Patient?name=${nameSearch}`);
      setPatients(response.entry || []);
      setOperationOutcome(null);
    } catch (err) {
      console.error('Search error:', err);
      setOperationOutcome(parseOperationOutcome(err));
    }
  };

  const handleLogout = async () => {
    setClient(null);
    setUsername('');
    setNameSearch('');
    setPatients([]);
    setSelectedPatient(null);
    setObservations([]);
    setOperationOutcome(null);

    localStorage.clear();
    sessionStorage.clear();

    window.location.href = '/';
  };

  // Helper function to format observation value
  const getObservationValue = (observation: Observation) => {
    if (observation.valueQuantity) {
      return `${observation.valueQuantity.value} ${observation.valueQuantity.unit || ''}`;
    }
    if (observation.valueString) {
      return observation.valueString;
    }
    if (observation.valueCodeableConcept) {
      return observation.valueCodeableConcept.text ||
        observation.valueCodeableConcept.coding?.[0]?.display ||
        'No value';
    }
    return 'No value';
  };

  return (
    <div className="p-4 max-w-4xl mx-auto">
      <div className="bg-white shadow rounded-lg">
        <div className="p-6">
          <div className="flex justify-between items-center mb-6">
            <h1 className="text-2xl font-bold">Patient Observation Viewer</h1>
            {client && (
              <div className="flex items-center gap-4">
                <span className="text-gray-600">
                  {username}
                </span>
                <button
                  onClick={handleLogout}
                  className="bg-red-500 text-white px-4 py-2 rounded hover:bg-red-600 focus:outline-none focus:ring-2 focus:ring-red-500 focus:ring-offset-2"
                >
                  Logout
                </button>
              </div>
            )}
          </div>

          {!client ? (
            <button
              onClick={initiateSmartAuth}
              className="bg-blue-500 text-white px-4 py-2 rounded hover:bg-blue-600 focus:outline-none focus:ring-2 focus:ring-blue-500 focus:ring-offset-2"
            >
              Login with SMART
            </button>
          ) : (
            <div className="space-y-6">
              <div className="flex gap-4">
                <form onSubmit={(e) => {
                  e.preventDefault();
                  searchPatients();
                }} className="flex gap-4 w-full">
                  <input
                    type="text"
                    placeholder="Search patient by name"
                    value={nameSearch}
                    onChange={(e) => setNameSearch(e.target.value)}
                    className="flex-1 border rounded px-3 py-2"
                  />
                  <button type="submit" className="bg-blue-500 text-white px-4 py-2 rounded">
                    Search Patients
                  </button>
                </form>
              </div>

              {selectedPatient && (
                <div className="bg-blue-50 border border-blue-200 rounded-lg p-4">
                  <div className="flex justify-between items-center">
                    <div>
                      <p className="text-sm text-blue-600">Selected Patient</p>
                      <p className="font-medium">
                        {selectedPatient.name?.[0]?.given?.join(' ')} {selectedPatient.name?.[0]?.family}
                      </p>
                      <p className="text-sm text-gray-500">ID: {selectedPatient.id}</p>
                    </div>
                    <button
                      onClick={() => {
                        setSelectedPatient(null);
                        setObservations([]);
                      }}
                      className="text-sm text-blue-600 hover:text-blue-800"
                    >
                      Change Patient
                    </button>
                  </div>
                </div>
              )}

              {patients.length > 0 && (
                <div className="border rounded-lg divide-y">
                  {patients.map(({ resource: patient }) => (
                    <div
                      key={patient.id}
                      onClick={() => handlePatientSelect(patient)}
                      className="p-4 hover:bg-gray-50 cursor-pointer"
                    >
                      <p className="font-medium">
                        {patient.name?.[0]?.given?.join(' ')} {patient.name?.[0]?.family}
                      </p>
                      <p className="text-sm text-gray-500">ID: {patient.id}</p>
                    </div>
                  ))}
                </div>
              )}

              {observations.length > 0 && (
                <div className="space-y-4">
                  <h2 className="text-xl font-semibold">Observations</h2>
                  <div className="border rounded-lg divide-y">
                    {observations.map(({ resource: obs }) => (
                      <div key={obs.id} className="p-4">
                        <div className="flex justify-between">
                          <div>
                            <p className="font-medium">
                              {obs.code.coding?.[0]?.display || obs.code.text || 'Unnamed Observation'}
                            </p>
                            <p className="text-sm text-gray-600">
                              Value: {getObservationValue(obs)}
                            </p>
                            <p className="text-sm text-gray-500">
                              Date: {obs.effectiveDateTime ? new Date(obs.effectiveDateTime).toLocaleDateString() : 'No date'}
                            </p>
                          </div>
                          <span className="text-sm text-gray-500">
                            Status: {obs.status}
                          </span>
                        </div>
                      </div>
                    ))}
                  </div>
                </div>
              )}

              {operationOutcome && (
                <div className="space-y-2">
                  {operationOutcome.issue.map((issue: any, index: number) => {
                    const severityColors = {
                      fatal: 'bg-red-100 border-red-500 text-red-900',
                      error: 'bg-red-50 border-red-400 text-red-700',
                      warning: 'bg-yellow-50 border-yellow-400 text-yellow-700',
                      information: 'bg-blue-50 border-blue-400 text-blue-700'
                    };

                    const severityIcons = {
                      fatal: '⛔',
                      error: '❌',
                      warning: '⚠️',
                      information: 'ℹ️'
                    };

                    return (
                      <div
                        key={index}
                        className={`p-4 rounded border ${severityColors[issue.severity] || severityColors.error}`}
                      >
                        <div className="flex items-start">
                          <span className="mr-2" role="img" aria-label={issue.severity}>
                            {severityIcons[issue.severity]}
                          </span>
                          <div>
                            <p className="font-medium capitalize">
                              {issue.severity} {issue.code && `(${issue.code})`}
                            </p>
                            <p className="mt-1">{issue.diagnostics}</p>
                          </div>
                        </div>
                      </div>
                    );
                  })}
                </div>
              )}
            </div>
          )}
        </div>
      </div>
    </div>
  );
};

export default PatientViewer;
