/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2025 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.smilecdr.consent;

import ca.cdr.api.consent.ConsentFetchQueries;
import ca.cdr.api.consent.ConsentLookupContext;
import ca.cdr.api.fhir.interceptor.CdrHook;
import ca.cdr.api.fhir.interceptor.CdrPointcut;
import ca.cdr.api.model.enm.PermissionEnum;
import ca.cdr.api.model.json.UserSessionDetailsJson;
import ca.uhn.fhir.context.FhirContext;
import ca.uhn.fhir.model.primitive.IdDt;
import ca.uhn.fhir.rest.api.server.RequestDetails;
import ca.uhn.fhir.rest.server.interceptor.consent.ConsentOutcome;
import ca.uhn.fhir.rest.server.interceptor.consent.IConsentContextServices;
import ca.uhn.fhir.rest.server.interceptor.consent.IConsentService;
import org.hl7.fhir.instance.model.api.IBaseResource;
import org.jetbrains.annotations.NotNull;
import org.springframework.web.util.UriComponentsBuilder;

import java.util.*;

public class DemoHooks {
	public static final String CONSENT_SERVICE_NAME = "consentDemoDefaultRule";
	private static final FhirContext myFhirContext = FhirContext.forR4Cached();

	@CdrHook(CdrPointcut.CONSENT_BUILD_CONSENT_SERVICE)
	IConsentService buildConsentService(String theName) {
		if (theName.equals(CONSENT_SERVICE_NAME)) {
			return new DemoDefaultConsentService();
		}
		return null;
	}

	@CdrHook(CdrPointcut.CONSENT_FETCH_QUERIES)
	public void getConsentFetchQueries(
			ConsentFetchQueries theConsentFetchQueries,
			RequestDetails theRequestDetails,
			ConsentLookupContext theConsentLookupContext,
			IBaseResource theResource,
			UserSessionDetailsJson theUserSessionDetails) {

		if (theUserSessionDetails.hasAuthority(PermissionEnum.ROLE_SUPERUSER)) {
			return;
		}

		Optional<IdDt> oPatientId = calculateConsentPatient(theResource);
		if (oPatientId.isEmpty()) {
			return;
		}
		IdDt patientId = oPatientId.get();

		// Search for Practitioner Consent resources based on the Practitioner value in the user's default launch
		// context
		theUserSessionDetails
				.getDefaultLaunchContextResourceIdForContextType("Practitioner")
				.ifPresent(practitionerReference -> {
					UriComponentsBuilder query = buildPatientQuery(patientId, theResource);
					query.queryParam("category", "INFA");
					query.queryParam("actor", practitionerReference);
					theConsentFetchQueries.addQuery(query.toUriString());
				});

		// Search for Organization Consent resources based on the Location value in the user's default launch context
		theUserSessionDetails
				.getDefaultLaunchContextResourceIdForContextType("Location")
				.ifPresent(locationReference -> {
					UriComponentsBuilder query = buildPatientQuery(patientId, theResource);
					query.queryParam("category", "IDSCL");
					query.queryParam("actor", locationReference);
					theConsentFetchQueries.addQuery(query.toUriString());
				});

		// Search for Patient Consent resources
		UriComponentsBuilder query = buildPatientQuery(patientId, theResource);
		query.queryParam("category", "59284-0");
		theConsentFetchQueries.addQuery(query.toUriString());
	}

	private @NotNull UriComponentsBuilder buildPatientQuery(IdDt thePatientId, IBaseResource theResource) {
		return UriComponentsBuilder.fromPath("Consent")
				.queryParam("status", "active")
				.queryParam("patient", thePatientId);
	}

	private Optional<IdDt> calculateConsentPatient(IBaseResource theResource) {
		final IllegalStateException error =
				new IllegalStateException("Unable to initialize consent because the patient configuration is missing.");

		if (theResource == null) {
			throw error;
		}
		return myFhirContext.newTerser().getCompartmentOwnersForResource("Patient", theResource, Set.of()).stream()
				.map(id -> new IdDt(id.getValue()))
				.filter(id -> Objects.equals("Patient", id.getResourceType()))
				.findFirst();
	}

	static class DemoDefaultConsentService implements IConsentService {
		@Override
		public ConsentOutcome canSeeResource(
				RequestDetails theRequestDetails,
				IBaseResource theResource,
				IConsentContextServices theContextServices) {
			if (theResource == null) {
				return ConsentOutcome.PROCEED;
			}

			// By default, we allow all resources to be seen

			return ConsentOutcome.AUTHORIZED;
		}
	}
}
